/*
 * Copyright (C) 2020 UBports foundation.
 * Author(s): Marius Gripsgard <marius@ubports.com>
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License version 3, as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
 * SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "platform.h"
#include "linux.h"
#ifdef HAVE_PROPS
# include "halium.h"
#endif

std::shared_ptr<Platform> Platform::create()
{
#ifdef HAVE_PROPS
    if (platform::Halium::usable())
        return std::make_shared<platform::Halium>();
#endif

    // Linux is our fallback
    return std::make_shared<platform::Linux>();
}
